package cn.com.qmth.print.manage.utils.excel;

import org.apache.poi.hssf.usermodel.HSSFCell;
import org.apache.poi.hssf.usermodel.HSSFDateUtil;
import org.apache.poi.openxml4j.exceptions.InvalidFormatException;
import org.apache.poi.ss.usermodel.*;

import java.io.IOException;
import java.io.InputStream;
import java.lang.reflect.Field;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Objects;

/**
 * 
 * @Description: 读取excel
 * @author ting.yin
 * @date 2016年8月19日
 */
public class ExcelReader extends ExcelUtils {

	public ExcelReader(Class<?> dataClass) {
		super(dataClass, true);
	}

	/**
	 * @param inputStream 输入流
	 * @param handle      单个对象处理器
	 * @return 错误信息集合
	 */
	public List<ExcelError> reader(InputStream inputStream,
								   ExcelReaderHandle handle) {
		List<ExcelError> excelErrors = new ArrayList<ExcelError>();
		try {
			Workbook wb = WorkbookFactory.create(inputStream);
			Sheet sheet = wb.getSheetAt(0);
			//校验表头信息
			Row row0 = sheet.getRow(0);
			for (int j = 0; j < this.getColumnSettings().size(); j++) {
				ColumnSetting columnSetting = this.getColumnSettings().get(j);
				Cell cell = row0.getCell(columnSetting.getIndex() - 1);
				String obj = String.valueOf(convert(cell));
				if (!Objects.equals(columnSetting.getHeader(), obj)) {
					ExcelError error = new ExcelError();
					error.setRow(1);
					error.setExcelErrorType("表头格式有误,请检查");
					excelErrors.add(error);
					return excelErrors;
				}
			}
			for (int i = 1; i <= sheet.getLastRowNum(); i++) {
				Row row = sheet.getRow(i);
				if (row == null) {
					return excelErrors;
				}
				Object dto = getDataClass().newInstance();
//				FieldAccess access = FieldAccess.get(getDataClass());
				try {
					for (int j = 0; j < this.getColumnSettings().size(); j++) {
						ColumnSetting columnSetting = this.getColumnSettings().get(j);
						Cell cell = row.getCell(columnSetting.getIndex() - 1);
						if (Objects.isNull(cell)) {
							// TODO: 2023/10/19 这个地方可以植入@NotNull判断必填报错 
							continue;
						}
						String value = analyzeExcelCellValue(cell);
						Field field = getDataClass().getDeclaredField(columnSetting.getFieldName());
						field.setAccessible(true);
						field.set(dto, value);
					}
				} catch (NoSuchFieldException e) {
					e.printStackTrace();
				} catch (SecurityException e) {
					e.printStackTrace();
				} catch (IllegalArgumentException e) {
					ExcelError error = new ExcelError();
					error.setRow(i + 1);
					error.setExcelErrorType("数据格式有误，请转成文本格式");
					excelErrors.add(error);
					return excelErrors;
				} catch (IllegalAccessException e) {
					e.printStackTrace();
				}
				ExcelError error = handle.handle(dto);
				if (error != null) {
					error.setRow(i + 1);
					excelErrors.add(error);
				}
			}
		} catch (InvalidFormatException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		} catch (Exception e) {
			e.printStackTrace();
		} finally {
			try {
				inputStream.close();
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
		return excelErrors;
	}

	private Object convert(Cell cell) {
		if (cell != null) {
			switch (cell.getCellType()) {
				case Cell.CELL_TYPE_STRING:
					return cell.getStringCellValue().toString();
				case Cell.CELL_TYPE_BOOLEAN:
					return String.valueOf(cell.getBooleanCellValue());
				case Cell.CELL_TYPE_NUMERIC:
					return Math.round(cell.getNumericCellValue());
			}
		}

		return null;
	}

	/**
	 * 解析excel单元格类型
	 *
	 * @param cell 单元格
	 * @return 转换成字符串后的值
	 */
	public static String analyzeExcelCellValue(Cell cell) {
		String cellValue;

		switch (cell.getCellType()) {
			case HSSFCell.CELL_TYPE_NUMERIC:
				if (HSSFDateUtil.isCellDateFormatted(cell)) {
					SimpleDateFormat sdf;
					// 验证short值
					if (cell.getCellStyle().getDataFormat() == 14) {
						sdf = new SimpleDateFormat("yyyy/MM/dd");
					} else if (cell.getCellStyle().getDataFormat() == 21) {
						sdf = new SimpleDateFormat("HH:mm:ss");
					} else if (cell.getCellStyle().getDataFormat() == 22) {
						sdf = new SimpleDateFormat("yyyy/MM/dd HH:mm:ss");
					} else {
						throw new RuntimeException("日期格式错误!!!");
					}
					Date date = cell.getDateCellValue();
					cellValue = sdf.format(date);
				} else {//处理数值格式
					cell.setCellType(HSSFCell.CELL_TYPE_STRING);
					cellValue = String.valueOf(cell.getRichStringCellValue().getString());
				}
				break;
			case HSSFCell.CELL_TYPE_STRING:
				cellValue = String.valueOf(cell.getStringCellValue());
				break;
			case HSSFCell.CELL_TYPE_BOOLEAN:
				cellValue = String.valueOf(cell.getBooleanCellValue());
				break;
			case HSSFCell.CELL_TYPE_FORMULA:
				cellValue = String.valueOf(cell.getCellFormula());
				break;
			case HSSFCell.CELL_TYPE_BLANK:
				cellValue = null;
				break;
			case HSSFCell.CELL_TYPE_ERROR:
				cellValue = "非法字符";
				break;
			default:
				cellValue = "未知类型";
				break;
		}
		return cellValue;
	}
}
